/***************************************************************************
 *   Copyright (C) 2006 by Simon Dible   *
 *   s.a.dible@lboro.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#define SOCKET "/tmp/usbb2k.sock"


#include "b2kinterface.h"
#include <qmessagebox.h>


B2kInterface::B2kInterface(QWidget* parentWindow){
	parent = parentWindow;
	started = false;
	phoneNumber = "";
	currentMode = USB;
}

B2kInterface::~B2kInterface(){
	

}

//connect to the usb device and then connect to skype by sending the name 
//command called from the constructor
bool B2kInterface::startUSBDevice(){
	//make a socket to talk to the usbb2k device 

	if ((s = socket(AF_UNIX, SOCK_STREAM, 0)) == -1) {
		QMessageBox::critical ( parent, "Error Connecting To Device Socket", "The program will now exit\nCheck the usb driver program is running.", QMessageBox::Yes, QMessageBox::NoButton, QMessageBox::NoButton );           
		return false;
	}
	remote.sun_family = AF_UNIX;
	strcpy(remote.sun_path, SOCKET);
	len = strlen(remote.sun_path) + sizeof(remote.sun_family);

	if (::connect(s, (struct sockaddr *)&remote, len) == -1) {
		QMessageBox::critical ( parent, "Error Finding Device", "The program will now exit.\nCheck the usb driver is running and that you have read write permissions on it.", QMessageBox::Yes, QMessageBox::NoButton, QMessageBox::NoButton );           
		return false;
	}
	//etat=fcntl(s,F_GETFL);
	etat = O_NONBLOCK;
	fcntl(s,F_SETFL,etat);
	emit b2kFeedbackMessage(new QString("Connected to usb device!\n"));
	sendb2kUSB();
	return true;

}

//change the state of the box to pstn 
void B2kInterface::sendb2kPSTN(){
	char str[50];
	strcpy(str,"SWITCH PSTN\n" );
	if (write(s, str,strlen(str)) == -1) {
		//perror("send");
		emit b2kFeedbackMessage(new QString("Send ERROR\n"));
	}
	currentMode = PSTN;
}

//change the state of the box to usb 
void B2kInterface::sendb2kUSB(){
	char str[50];
	strcpy(str,"SWITCH USB\n" );
	if (write(s, str,strlen(str)) == -1) {
		//perror("send");
		emit b2kFeedbackMessage(new QString("Send ERROR\n"));
	}
	currentMode = USB;
}

//turn on the phone ringer 
void B2kInterface::startRing(){
	char str[50];
	strcpy(str,"RING 1\n");
	if (write(s, str,strlen(str)) == -1) {
		//perror("send");
		emit b2kFeedbackMessage(new QString("Send ERROR\n"));
	}
}

//turn off the phon ringer
void B2kInterface::stopRing(){
	char str[50];
	strcpy(str,"RING 0\n" );
	if (write(s, str,strlen(str)) == -1) {
		//perror("send");
		emit b2kFeedbackMessage(new QString("Send ERROR\n"));
	}
}

//start or stop the thread from running 
void B2kInterface::setStart(bool b){
		started = b;
	}

//start listening to the socket and send a signal when you get somthing 
void B2kInterface::run (){
	char str[101];
	QString* qstr;
	while (started){
		if ((t=read(s, str,100)) >= 0) {
			str[t] = '\0';
			if (t>0) {
				processCommands(str);
				qstr = new QString(str);
				if (qstr->contains("HANDSET ON") || qstr->contains("HANDSET OFF")) 
					emit b2kmessage(qstr);
				//fprintf(stderr,"box said:%s", str);
				usleep(50);
			}
		}
		usleep(50);
	}
}

//take some actions based up on what keys you press
// currently changing mode doesn;t work from the keypad 
void B2kInterface::processCommands(char* s){
	QString q = QString(s);
	
	if(q.contains("USB DEVICE INIT ERROR", false)){
		fprintf(stderr, "Stratup error, connection restarted !\n");
		startUSBDevice();
	}
	else if(q.contains("HANDSET ON", false)){
			phoneNumber = "";
		emit b2kmessage(new QString("FOCUS"));
	}
	else if(q.contains("HANDSET OFF", false)){
			phoneNumber = "";
	}
	else if(q.contains("KEY  01", false)){
			phoneNumber.append ("1");
	}
	else if(q.contains("KEY  02", false)){
			phoneNumber.append ("2");
	}
	else if(q.contains("KEY  03", false)){
			phoneNumber.append ("3");
	}
	else if(q.contains("KEY  04", false)){
			phoneNumber.append ("4");
	}
	else if(q.contains("KEY  05", false)){
			phoneNumber.append ("5");
	}
	else if(q.contains("KEY  06", false)){
			phoneNumber.append ("6");
	}
	else if(q.contains("KEY  07", false)){
			phoneNumber.append ("7");
	}
	else if(q.contains("KEY  08", false)){
			phoneNumber.append ("8");
	}
	else if(q.contains("KEY  09", false)){
			phoneNumber.append ("9");
	}
	else if(q.contains("KEY  00", false)){
			phoneNumber.append ("0");
	}
	//star key not sure what to do with this 
	else if(q.contains("KEY  0b", false)){
			phoneNumber.append ("*");
	}	
	//hash key place the call 
	else if(q.contains("KEY  0c", false)){
		//if dialed ** then switch mode
		//it doesn't switch back there might be somthin in hardware for the other line
		//i don;t have a phone line so can;t play about can someone else?
		if (phoneNumber == "**"){
			phoneNumber = "";
			if (currentMode == USB){
				sendb2kPSTN();
				QString* send = new QString("USBSTATE PSTN");
				emit b2kmessage(send);
			}else{
				sendb2kUSB();
				QString* send = new QString("USBSTATE USB");
				emit b2kmessage(send);
			}
			return;
		}
		QString* send = new QString("CALL ");
		send->append(phoneNumber);
		emit b2kmessage(send);
		phoneNumber = "";
	}
	else if(q.contains("USB DEVICE FOUND", false)){
		//do nothing 
	}
	else{
		printf("UNKNOWN KEY:%s",q.ascii());
	}
	
}

