// 
// Copyright (c) 2004-2006, Skype Limited.
// All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 
//   * Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above
//     copyright notice, this list of conditions and the following
//     disclaimer in the documentation and/or other materials provided
//     with the distribution.
//   * Neither the name of the Skype Limited nor the names of its
//     contributors may be used to endorse or promote products derived
//     from this software without specific prior written permission.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
// FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
// COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
// INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
// BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
// ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
// POSSIBILITY OF SUCH DAMAGE.
// 

//most of this is written by simon dible and copies the original version by skype
//I release my work under the gpl license, see the original version for changes.
/***************************************************************************
 *   Copyright (C) 2006 by Simon Dible   *
 *   s.a.dible@lboro.ac.uk   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "xmessages.h"
#include "skypeapidlg.h"

#include <qlineedit.h>
#include <qtextedit.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qlabel.h>

#include <X11/Xlib.h>
#include <X11/Xatom.h>


#include <qradiobutton.h>
#include <qmessagebox.h>
#include <qstringlist.h>





static const char *skypemsg = "SKYPECONTROLAPI_MESSAGE";

SkypeAPI::SkypeAPI(QWidget *parent)	: Skype_API(parent)
{
	msg = new XMessages(skypemsg, this);

	//wether we have connected to the skype window or not 
	started = false;

	//the b2k interface 
	b = new B2kInterface(this);

	//event handeling remove allot of this later
//	connect(sendButton, SIGNAL(clicked()), this, SLOT( sendMessage()));
//	connect(commandLine, SIGNAL(returnPressed()), this, SLOT(sendMessage()));
	connect(msg, SIGNAL(gotMessage(int, const QString &)), this, SLOT(gotMessage(int, const QString &)));
	connect(radioButton1, SIGNAL(clicked()),b,SLOT(sendb2kPSTN()));
	connect(radioButton2, SIGNAL(clicked()),b,SLOT(sendb2kUSB()));
	connect (b, SIGNAL(b2kFeedbackMessage(QString*)), this, SLOT(appendToTextArea(QString*)));
	connect (b, SIGNAL(b2kmessage(QString*)), this, SLOT(gotB2KMessage(QString*)));

	//the id of the call assigned by skype
	callNumber = QString("");

	//look for the skype window 
	detectSkype();

	//connect tot the box or exit if you fail 
	if (!b->startUSBDevice()){
		exit(1);
	}

	//connect to skype
	sendMessage( new QString("NAME B2K CONNECTOR"));

	//start listening to events from the box
	b->setStart(true);
	b->start();

	//define some defaults
	incall = false;
	ringing = false;
}

void SkypeAPI::detectSkype()
{
	Atom skype_inst = XInternAtom(qt_xdisplay(), "_SKYPE_INSTANCE", True);

	Atom type_ret;
	int format_ret;
	unsigned long nitems_ret;
	unsigned long bytes_after_ret;
	unsigned char *prop;
	int status;

	status = XGetWindowProperty(qt_xdisplay(), qt_xrootwin(), skype_inst, 0, 1, False, XA_WINDOW, &type_ret, &format_ret, &nitems_ret, &bytes_after_ret, &prop);

	// sanity check
	if(status != Success || format_ret != 32 || nitems_ret != 1)
	{
		skype_win = (WId)-1;
		logWindow->append(QString("<font color=blue>Skype not detected, status %1</font>\n").arg(status));
		QMessageBox::critical ( this, "Error Connecting to Skype", "Couldn;t find the main window\nStart Skype then restart this application.", QMessageBox::Yes, QMessageBox::NoButton, QMessageBox::NoButton );           
		exit(1);
	}
	else
	{
		skype_win = * (const unsigned long *) prop & 0xffffffff;
		logWindow->append(QString("<font color=blue>Skype instance found, window id %1</font>\n").arg(skype_win));
	}
}

//added by simon dible to send messages 
void SkypeAPI::sendMessage(QString* command)
{
char a[100];
	strcpy(a, command->ascii());
	a[strlen(a)] = '\0';
	if(!msg->sendMessage(skype_win, skypemsg, a))
		logWindow->append("<font color=red>Sending failed</font>");
	else
	{
	//	logWindow->append("<font color=green>" + commandLine->text() + "</font>\n");
	//	commandLine->clear();
	}
}


//mostly rewritten to listen for specific events from skype !
void SkypeAPI::gotMessage(int win, const QString &message)
{
	if(win != (int)skype_win)
		qDebug("Message coming from different skype instance :^)");

	//if in startup state look for connected to skyp message 
	if (!started){
		if (strcmp(message,"ERROR 1") == 0){
			//skype refused a connection so quit out 
		QMessageBox::critical ( this, "Error Connecting To Skype", "Couldn't connect to skype please restart and try again.", QMessageBox::Yes, QMessageBox::NoButton, QMessageBox::NoButton );           
		exit(1);
		}else if(strcmp(message,"OK") == 0){
			sendMessage(new QString("PROTOCOL 5"));
			started = true;
		}
		//logWindow->append("<font color=red>" + message + "</font>\n");
	}
//#############################################################
//			we are in full working mode Look for message ststus
//#############################################################
	else{
		// LINE BELOW SHOWS SKYPE RETURN MESSAGES IN WINDOW
		//logWindow->append("<font color=red>" + message + "</font>\n");


		//#####################################
		//see if we have an incomming call 
		//#####################################
		if (message.contains("CALL",false ) && message.contains("STATUS RINGING",false )){
			//we have a call so start ringing the box 

			//tokenise the reply to get the call number 
			QStringList l = QStringList::split(' ', message); 
			callNumber = l[1];
			//make the box ring 
			b->startRing();
			//set flags
			ringing = true;
			incall = false;
		}
		//#####################################
		//see if we have refused a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS REFUSED",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			ringing = false;
			incall = false;
		}
		//#####################################
		//see if we have FAILED a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS FAILED",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			ringing = false;
			incall = false;
		}
		//#####################################
		//see if we have finished a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS FINISHED",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			ringing = false;
			incall = false;
		}
		//#####################################
		//see if we have canceled a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS CANCELLED",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			ringing = false;
			incall = false;
		}
		//#####################################
		//see if we have MISSED a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS MISSED",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			ringing = false;
			incall = false;
		}
		//#####################################
		//see if we have a call being produced and grab the calll id 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS ROUTING",false )){
			//we have a call so start ringing the box 
			incall = true;
			QStringList l = QStringList::split(' ', message); 
			callNumber = l[1];
		}
		//#####################################
		//see if we have BUSY a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS BUSY",false )){
			//we have a call so start ringing the box 
			incall = false;
		}
		//#####################################
		//see if we have answered a call 
		//#####################################
		else if (message.contains("CALL",false ) && message.contains("STATUS INPROGRESS",false )){
			//we have a call so start ringing the box 
			b->stopRing();
			//get the call number from return string 
			QStringList l = QStringList::split(' ', message); 
			callNumber = l[1];
			//set flags
			ringing = false;
			incall = true;
		}
	}
}


//send commands from the box to skype 
void SkypeAPI::gotB2KMessage(QString* message){
	//###########################################
	//		You put down the phone 
	//###########################################
	if (message->contains("HANDSET OFF")){
		//if we are in a call hang up !
		if (incall == true){
			sendMessage(new QString("SET CALL " + callNumber +  " STATUS FINISHED"));
		}
		sendMessage(new QString("MINIMIZE"));
		incall = false;
	}
	//###########################################
	//		You pick up the phone 
	//###########################################
	else if (message->contains("HANDSET ON")){
		//if we are in a call hang up !
		if (ringing == true){
			incall = true;
			ringing = false;
			b->stopRing();
			sendMessage(new QString("SET CALL " + callNumber +  " STATUS INPROGRESS"));
		}
	}
	//###########################################
	//		make a call 
	//###########################################
	else if (message->contains("CALL ") ){
			if (!incall){
				incall = true;
				sendMessage(message);
			}
	}
	//###########################################
	//		focus the main window
	//###########################################
	else if (message->contains("FOCUS") ){
			sendMessage(message);
	}
	//###########################################
	//		you are changing from pstn to usb
	//###########################################
	else if (message->contains("USBSTATE") ){
		QStringList l = QStringList::split(' ', *message); 
		if ( l[1] == "USB"){
			radioButton2->setChecked(true) ;
		}else{
			radioButton1->setChecked(true) ;
		}
	}

}


//this is here for debug messages 
void SkypeAPI::appendToTextArea(QString* qstr){
		logWindow->append(*qstr);
		logWindow->update();
}





